@@ -1,6 +1,6 @@
 ;;; lang/c99/cppmach.scm - CPP expression grammar
 
-;; Copyright (C) 2015,2016,2018 Matthew R. Wette
+;; Copyright (C) 2015,2016,2018,2026 Matthew Wette
 ;;
 ;; This library is free software; you can redistribute it and/or
 ;; modify it under the terms of the GNU Lesser General Public
@@ -27,8 +27,10 @@
 
 (define cpp-spec
   (lalr-spec
-   (notice (string-append "Copyright (C) 2016,2017 Matthew R. Wette"
+   (notice (string-append "Copyright (C) 2016,2017,2026 Matthew Wette"
                           license-lgpl3+))
+   (reserve "..." ";" "|=" "^=" "&=" ">>=" "<<=" "%=" "/=" "*=" "-="
+            "+=" "=" "->" "." "]" "[" "}" "{") ;; same chseq's for C99 & CPP !
    (expect 0)
    (start conditional-expression)
    (grammar
