@@ -1,6 +1,6 @@
 ;;; lang/c99/mach.scm - C parser grammer
 
-;; Copyright (C) 2015-2025 Matthew Wette
+;; Copyright (C) 2015-2026 Matthew Wette
 ;;
 ;; This library is free software; you can redistribute it and/or
 ;; modify it under the terms of the GNU Lesser General Public
@@ -306,6 +306,7 @@
      ("unsigned" "__int128" ($$ '(fixed-type "unsigned __int128")))
      ;; gnu extensions
      ("__int128_t" ($$ '(fixed-type "__int128")))
+     ("__uint128_t" ($$ '(fixed-type "unsigned __int128"))) ;; not documented
      ("unsigned" "__int128_t" ($$ '(fixed-type "unsigned __int128")))
      ;; fixes 250627
      ("short" "signed" ($prec 'imp) ($$ '(fixed-type "short")))
@@ -324,8 +325,10 @@
      ("double" ($prec 'imp) ($$ '(float-type "double")))
      ("long" "double" ($$ '(float-type "long double")))
      ("_Float16" ($$ '(float-type "_Float16")))
+     ("_Float32" ($$ '(float-type "float")))
+     ("_Float64" ($$ '(float-type "double")))
      ("_Float128" ($$ '(float-type "_Float128")))
-     ("_float16" ($$ '(float-type "_Float16")))
+     ;; gnu extensions
      ("_float128" ($$ '(float-type "_Float128"))))
 
     (complex-type-specifier
@@ -906,12 +909,7 @@
     (code-comment ($code-comm ($$ `(comment ,$1))))
     (lone-comment ($lone-comm ($$ `(comment ,$1))))
     (cpp-statement ('cpp-stmt ($$ `(cpp-stmt ,$1))))
-    (pragma
-     ($pragma ($$ `(pragma ,$1)))
-     ;;("_Pragma" "(" string-literal ")" ($$ `(pragma ,$3)))
-     )
-
-    )))
+    (pragma ($pragma ($$ `(pragma ,$1)))))))
 
 ;;; === parsers =========================
 
