@@ -34,8 +34,14 @@
   #:use-module (srfi srfi-2)            ; and-let*
   #:use-module (sxml fold)
   #:use-module (ice-9 popen)            ; gen-gcc-cpp-defs
-  #:use-module (ice-9 rdelim)           ; gen-gcc-cpp-defs
-  #:use-module (ice-9 match))
+  #:use-module (ice-9 rdelim))           ; gen-gcc-cpp-defs
+(cond-expand
+ (mes
+  (use-modules (smatch))
+  (define-macro (match exp . clauses)
+    `((smatch-lambda . ,clauses) ,exp)))
+(else
+  (use-modules (ice-9 match))))
 
 (define c99-def-help
   (let ((base
@@ -44,6 +50,8 @@
             "__inline__=inline" "__inline=__inline__"
             "__restrict__=restrict" "__restrict=__restrict__"
             "__signed__=signed" "__signed=__signed__"
+            "__has_include(X)=__has_include__(#X)"
+            "__has_include_next(X)=__has_include_next__(#X)"
             "asm(X)=__asm__(X)" "__asm(X)=__asm__(X)"
             "__attribute(X)=__attribute__(X)"
             "__volatile__=volatile" "__volatile=__volatile__"
