@@ -1,6 +1,6 @@
 ;;; nyacc/lex.scm
 
-;; Copyright (C) 2015-2022 - Matthew R.Wette
+;; Copyright (C) 2015-2022,2025,2026 Matthew Wette
 ;; 
 ;; This library is free software; you can redistribute it and/or modify it
 ;; under the terms of the GNU Lesser General Public License as published by
@@ -46,12 +46,13 @@
             read-c-ident read-c$-ident
             read-c-comm
             read-c-string
-            read-c-chlit
+            read-c-chlit read-c-mclit
             read-c-num
             read-oct read-hex
             like-c-ident? like-c$-ident?
             c-escape
             cnumstr->scm
+            chseq->canon-map chseq->canon-unmap
             filter-mt remove-mt map-mt make-ident-like-p
             c:ws c:if c:ir)
   #:use-module ((srfi srfi-1) #:select (remove append-reverse)))
@@ -132,7 +133,7 @@
             #t)))
         #f)))
          
-;; @deffn {Procedure} skip-c-space ch => #f|#t
+;; @deffn {Procedure} skip-c-space ch => #f | #t
 ;; If @code{ch} is space, skip all spaces, then return @code{#t}, else
 ;; return @code{#f}.
 ;; @end deffn
@@ -169,10 +170,12 @@
 (define (make-ident-like-p reader)
   (lambda (s) (and (string? s)
                    (positive? (string-length s))
-                   (eval-reader reader s)
+                   (with-input-from-string s
+                     (lambda ()
+                       (and (reader (read-char))
+                            (eof-object? (read-char)))))
                    #t)))
 
-
 ;; @deffn {Procedure} make-ident-keyword-reader ident-reader match-table [tval]
 ;; Generate a procedure from an ident reader and a parser match-table
 ;; that takes a character and returns @code{#f} or a pair for the parser.
@@ -242,7 +245,7 @@
 ;;(export new-make-string-reader)
 ;; string-literal -> parser -> pretty-print => string-literal
 
-;; @deffn {Procedure} read-oct => 123|#f
+;; @deffn {Procedure} read-oct => 123 | #f
 ;; Read octal number, assuming @code{\0} have already been read.
 ;; Return integer.
 ;; @end deffn
@@ -257,7 +260,7 @@
           (loop (+ (* 8 cv) (- (char->integer ch) 48)) (read-char) (1+ n)))
          (else (unread-char ch) cv))))))
 
-;; @deffn {Procedure} read-hex => 123|#f
+;; @deffn {Procedure} read-hex => 123 | #f
 ;; Read hex number.  Assumes prefix (e.g., "0x" has already been read).
 ;; Returns integer.
 ;; @end deffn
@@ -323,6 +326,26 @@
 ;; @end deffn
 (define (make-chlit-reader . rest) (error "NOT IMPLEMENTED"))
 
+(define (read-esc-char)
+  (let ((c2 (read-char)))
+    (case c2
+      ((#\t) #\tab)               ; horizontal tab U+0009
+      ((#\n) #\newline)           ; newline U+000A
+      ((#\v) #\vtab)              ; verticle tab U+000B
+      ((#\f) #\page)              ; formfeed U+000C
+      ((#\r) #\return)            ; return U+000D
+      ((#\a) #\alarm)             ; alert U+0007
+      ((#\b) #\backspace)         ; backspace U+0008 not in guile 1.8
+      ((#\0) (integer->char (read-oct))) ; octal
+      ((#\1 #\2 #\3 #\4 #\5 #\6 #\7)     ; octal
+       (unread-char c2) (integer->char (read-oct)))
+      ((#\x) (integer->char (read-hex))) ; hex
+      ((#\\ #\' #\" #\? #\|) c2)
+      (else (error "bad escape sequence" c2)))))
+
+(define (wchar t)
+  (case t ((#\L) '$chlit/L) ((#\u) '$chlit/u) ((#\U) '$chlit/U)))
+
 ;; @deffn {Procedure} read-c-chlit ch
 ;; @example
 ;; ... 'c' ... => (read-c-chlit #\') => '($chlit . #\c)
@@ -331,28 +354,10 @@
 ;; @code{$chlit/u} for @code{char16_t}, or @code{$chlit/U} for @code{char32_t}.
 ;; @end deffn
 (define (read-c-chlit ch)
-  (define (read-esc-char)
-    (let ((c2 (read-char)))
-      (case c2
-        ((#\t) "\t")               ; horizontal tab U+0009
-        ((#\n) "\n")               ; newline U+000A
-        ((#\v) "\v")               ; verticle tab U+000B
-        ((#\f) "\f")               ; formfeed U+000C
-        ((#\r) "\r")               ; return U+000D
-        ((#\a) "\x07")             ; alert U+0007
-        ((#\b) "\x08")             ; backspace U+0008 not in guile 1.8
-        ((#\0) (string (integer->char (read-oct)))) ; octal
-        ((#\1 #\2 #\3 #\4 #\5 #\6 #\7)              ; octal
-         (unread-char c2) (string (integer->char (read-oct))))
-        ((#\x) (string (integer->char (read-hex)))) ; hex
-        ((#\\ #\' #\" #\? #\|) (string c2))
-        (else (error "bad escape sequence" c2)))))
-  (define (wchar t)
-    (case t ((#\L) '$chlit/L) ((#\u) '$chlit/u) ((#\U) '$chlit/U)))
   (cond
    ((char=? ch #\')
     (let* ((c1 (read-char))
-           (sc (if (eqv? c1 #\\) (read-esc-char) (string c1))))
+           (sc (string (if (eqv? c1 #\\) (read-esc-char) c1))))
       (if (not (char=? #\' (read-char)))
           (throw 'nyacc-error "read-c-chlit: bad char literal"))
       (cons '$chlit sc)))
@@ -364,6 +369,25 @@
        (else (unread-char c1) #f))))
    (else #f)))
 
+;; read multi-char literal : braindamage in gobject/glib-types.h
+;; IIRC only pango uses this as of 2026
+(define (read-c-mclit ch)
+  (define (readit ch)
+    (let loop ((chl '()) (ch ch))
+      (cond
+       ((char=? ch #\') (cons '$chlit (rls chl)))
+       ((char=? ch #\\) (loop (cons (read-esc-char) chl) (read-char)))
+       (else (loop (cons ch chl) (read-char))))))
+  (cond
+   ((char=? ch #\') (readit (read-char)))
+   ((char-set-contains? c:cx ch)
+    (let ((c1 (read-char)))
+      (cond
+       ((eof-object? c1) #f)
+       ((char=? c1 #\') (cons (wchar ch) (cdr (read-c-chlit c1))))
+       (else (unread-char c1) #f))))
+   (else #f)))
+
 (define (fix-dot l) (if (char=? #\. (car l)) (cons #\0 l) l))
 
 ;; @deffn {Procedure} read-basic-num ch [#:signed #t] => #f|pair
@@ -587,11 +611,17 @@
         ((char-numeric? ch) (loop (cons ch chl) ty 50 (read-char)))
         ((memq ch '(#\F #\f)) (loop (cons ch chl) ty 69 (read-char)))
         ((memq ch '(#\L #\l)) (loop (cons ch chl) ty 65 (read-char)))
+        ((memq ch '(#\B #\b)) (loop (cons ch chl) ty 51 (read-char)))
         ((memq ch '(#\U #\u)) (loop (cons ch chl) '$fixpt 66 (read-char)))
         ((memq ch '(#\K #\R #\k #\r))
          (loop (cons ch chl) '$fixpt 70 (read-char)))
         ((memq ch '(#\D #\d)) (loop (cons ch chl) '$decfl 68 (read-char)))
         (else (loop chl ty 70 ch))))
+      ((51) ;; got B looking for BF16 suffix
+       (cond
+        ((eof-object? ch) (loop chl ty 72 ch))
+        ((memq ch '(#\F #\f)) (loop (cons ch chl) ty 69 (read-char)))
+        (else (bad-sfx chl))))
       ;; suffixes
       ((61) ;; fixed/u
        (cond
@@ -642,13 +672,13 @@
         ((memq ch '(#\K #\R #\k #\r))
          (loop (cons ch chl) '$fixpt 70 (read-char)))
         (else (bad-sfx chl))))
-      ((68) ;; got (d|D), look for dDfFlL
+      ((68) ;; got (d|D), look for dDfFlL or 0-9
        (cond
         ((eof-object? ch) (bad-sfx chl))
-        ((memq ch '(#\D #\F #\L #\d #\f #\l))
-         (loop (cons ch chl) '$decfl 72 ch))
+        ((memq ch '(#\D #\d #\F #\f #\L #\l)) (loop (cons ch chl) '$decfl 72 ch))
+        ((char-numeric? ch) (loop (cons ch chl) ty 69 (read-char)))
         (else (bad-sfx chl))))
-      ((69) ;; bizarre gcc float suffixes: F128 F32x
+      ((69) ;; special float suffixes: F128 F32x F16 BF16
        (cond
         ((eof-object? ch) (loop chl ty 72 ch))
         ((char-numeric? ch) (loop (cons ch chl) ty 69 (read-char)))
@@ -673,29 +703,30 @@
 ;; This probably belongs in @code{(nyacc lang util)}.
 ;; @end deffn
 (define (cnumstr->scm str)
-  (define (2- n) (1- (1- n))) (define (3- n) (1- (2- n)))
+  (define (2- n) (1- (1- n)))
+  (define (3- n) (1- (2- n)))
+  (define (trim-rt st nd) ;; trim U, UL, ULL (and lowercase) from right
+    (if (char-set-contains? c:sx (string-ref str (1- nd)))
+        (if (char-set-contains? c:sx (string-ref str (2- nd)))
+            (if (char-set-contains? c:sx (string-ref str (3- nd)))
+                (substring str st (3- nd))
+                (substring str st (2- nd)))
+            (substring str st (1- nd)))
+        (substring str st nd)))
   (let* ((nd (string-length str)))
-    (define (trim-rt st) ;; trim U, UL, ULL (and lowercase) from right
-      (if (char-set-contains? c:sx (string-ref str (1- nd)))
-          (if (char-set-contains? c:sx (string-ref str (2- nd)))
-              (if (char-set-contains? c:sx (string-ref str (3- nd)))
-                  (substring str st (3- nd))
-                  (substring str st (2- nd)))
-              (substring str st (1- nd)))
-          (substring str st nd)))
     (if (< nd 2) str
         (if (char=? #\0 (string-ref str 0))
             (cond
              ((char=? #\x (string-ref str 1))
-              (string-append "#x" (trim-rt 2)))
+              (string-append "#x" (trim-rt 2 nd)))
              ((char=? #\X (string-ref str 1))
-              (string-append "#x" (trim-rt 2)))
+              (string-append "#x" (trim-rt 2 nd)))
              ((char=? #\b (string-ref str 1))
-              (string-append "#b" (trim-rt 2)))
+              (string-append "#b" (trim-rt 2 nd)))
              ((char-numeric? (string-ref str 1))
-              (string-append "#o" (trim-rt 1)))
-             (else (trim-rt 0)))
-            (trim-rt 0)))))
+              (string-append "#o" (trim-rt 1 nd)))
+             (else (trim-rt 0 nd)))
+            (trim-rt 0 nd)))))
 
 ;;.@deffn {Procedure} si-map string-list ix => a-list
 ;; Convert list of strings to alist of char at ix and strings.
@@ -715,6 +746,29 @@
       (loop (cons (cons (string-ref (car sl) ix) (list (car sl))) sal)
             (cdr sl))))))
 
+(define (chseq->symbol cs)
+  (let* ((iseq (string-fold (lambda (c s) (cons* (char->integer c) s)) '() cs))
+         (guts (string-join (map (lambda (i) (number->string i 16)) iseq) "%"))
+         (name (string-append "%" guts)))
+    (string->symbol name)))
+(export chseq->symbol) ;; for debugging
+
+;; @deffn chseq->canon-map cst
+;; @deffnx chseq->canon-unmap cst
+;; The first procedure maps a char-sequence alist to one mapping the string
+;; form (e.g., @code{"+="}) to a canonical form (e.g. @code{_2B_3D_}).
+;; The second generates an alist with the canonical form keys and values
+;; from @var{cst}.  So, for example, pairs from @var{cst} are transformed as
+;; @example
+;; '("+=" . 123) -> '("+=" . %2B%3D)  '(%2B%3D . 123)
+;; @example
+;; @end deffn
+(define (chseq->canon-map cst)
+  (map (lambda (pair) (cons (car pair) (chseq->symbol (car pair)))) cst))
+(define (chseq->canon-unmap cst)
+  "See chseq"
+  (map (lambda (pair) (cons (chseq->symbol (car pair)) (cdr pair))) cst))
+
 ;;.@deffn {Procedure} make-tree strtab -> tree
 ;; This routine takes an alist of strings and symbols and makes a tree
 ;; that parses one char at a time and provide @code{'else} entry for
@@ -759,14 +813,14 @@
           #f))))))
 
 ;; @deffn {Procedure} make-comm-reader comm-table [#:eat-newline #t] => \
-;;   ch bol -> ('$code-comm "..")|('$lone-comm "..")|#f
+;;   ch bol -> ('$code-comm "..")|('$lone-comm "..") | #f
 ;; comm-table is list of cons for (start . end) comment.
 ;; e.g. ("--" . "\n") ("/*" . "*/")
 ;; test with "/* hello **/"
 ;; If @code{eat-newline} is specified as true then for read comments 
 ;; ending with a newline a newline swallowed with the comment.
 ;; The returned procedure has signature
-;; @code{(proc ch #:optional bol #:skip-prefix #t|#f)}
+;; @code{(proc ch #:optional bol #:skip-prefix #t | #f)}
 ;; @* Note: assumes backslash is never part of the end
 ;; @end deffn
 (define* (make-comm-reader comm-table #:key eat-newline)
